SystemOrganization addCategory: #'Pacman-Cells'!
SystemOrganization addCategory: #Pacman!

StringButtonMorph subclass: #MyStartButton
	instanceVariableNames: 'game'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pacman'!

!MyStartButton methodsFor: 'initialization' stamp: 'cl 6/19/2012 13:25'!
game: aGame
	game := aGame.! !

!MyStartButton methodsFor: 'event handling' stamp: 'cl 6/19/2012 13:22'!
handlesMouseUp: anEvent
	^true.! !

!MyStartButton methodsFor: 'initialization' stamp: 'cl 6/19/2012 13:20'!
initialize
	super initialize.
	self contents: 'start Game!!'.
	self position: 220@120.! !

!MyStartButton methodsFor: 'event handling' stamp: 'cl 6/19/2012 13:39'!
mouseUp: anEvent
	game btnStartPressed.! !

Morph subclass: #PacmanCell
	instanceVariableNames: 'type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pacman-Cells'!

PacmanCell subclass: #DotCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pacman-Cells'!

!DotCell methodsFor: 'initialization' stamp: 'cl 6/19/2012 10:47'!
initialize
	super initialize.
	type := #DOT.
	self color: Color lightYellow.! !

!PacmanCell methodsFor: 'initialization' stamp: 'cl 6/19/2012 11:06'!
initialize
	"initialiser for PacmanCell"
	super initialize.
	self width: 20.
	self height: 20.
	self color: Color white.
	type := #BLANK.! !

PacmanCell subclass: #PlayerCell
	instanceVariableNames: 'uniqueId direction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pacman-Cells'!

!PlayerCell methodsFor: 'logic' stamp: 'cl 6/19/2012 14:10'!
direction: aDirection
	"sets the direction this player should move"
	(aDirection == #UP) ifTrue: [
			"UP"
			direction := #UP.
			^self.
		].
	(aDirection == #DOWN) ifTrue: [
			"DOWN"
			direction := #DOWN.
			^self.
		].
	(aDirection == #LEFT) ifTrue: [
			"LEFT"
			direction := #LEFT.
			^self.
		].
	(aDirection == #RIGHT) ifTrue: [
			"RIGHT"
			direction := #RIGHT.
			^self.
		].
	(aDirection == #NO) ifTrue: [
			"NO direction"
			direction := #NO.
			^self.
		].! !

!PlayerCell methodsFor: 'initialization' stamp: 'cl 6/19/2012 13:55'!
initialize
	super initialize.
	type := #PLAYER.
	direction := #NO.! !

!PlayerCell methodsFor: 'logic' stamp: 'cl 6/19/2012 14:07'!
move
	"moves the player cell in his current direction"
	(direction == #UP) ifTrue: [
			"UP"
			^self.
		].
	(direction == #DOWN) ifTrue: [
			"DOWN"
			^self.
		].
	(direction == #LEFT) ifTrue: [
			"LEFT"
			^self.
		].
	(direction == #RIGHT) ifTrue: [
			"RIGHT"
			^self.
		].
	"else do nothing"! !

!PlayerCell methodsFor: 'initialization' stamp: 'cl 6/19/2012 13:55'!
uniqueId: anInteger
	"sets the unique ID of this player cell"
	uniqueId := anInteger.! !

PacmanCell subclass: #WallCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pacman-Cells'!

!WallCell methodsFor: 'initialization' stamp: 'cl 6/19/2012 10:40'!
initialize
	super initialize.
	type := #WALL.
	self color: Color black.! !

Object subclass: #PacmanPlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pacman'!

Object subclass: #WorldParser
	instanceVariableNames: 'game'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pacman'!

!WorldParser methodsFor: 'initialization' stamp: 'cl 6/18/2012 18:17'!
addGame: aGame
	"adds Game to this parser"
	game := aGame.! !

!WorldParser methodsFor: 'initialization' stamp: 'cl 6/18/2012 20:10'!
initialize
	super initialize.
	game := nil.! !

!WorldParser methodsFor: 'as yet unclassified' stamp: 'cl 6/19/2012 14:38'!
parseFileAt: filePath
	"parses the file at given path and returns a cell matrix"
	|file content cells i|
	file := (FileStream fileNamed: filePath) readOnly.
	
	content := file contentsOfEntireFile.
	
	i := 1.
	[i <= content lineCount] whileTrue: [
		|currentLine j c|
		currentLine := content lineNumber: i.
		"when accessing the first line create the matrix"
		(i == 1) ifTrue: [
			cells := Matrix rows: content lineCount columns: currentLine size.
			].
		
		"iterating over each character of the line"
		j := 1.
		[j <= currentLine size.] whileTrue: [
				|char|
				char := Character value: (currentLine byteAt: j).
				
				(char == $#) ifTrue: [
						"wall"
						c := WallCell new.
					] ifFalse: [
						(char == $.) ifTrue: [
								"dot"
								c := DotCell new.
							] ifFalse: [
								(char == $1) ifTrue: [
										"player 1"
										c := PlayerCell new.
										c color: Color red.
										game addPlayer: c WithId: 1.
									] ifFalse: [
										(char == $2) ifTrue: [
												"player 2"
												c := PlayerCell new.
												c color: Color blue.
												game addPlayer: c WithId: 2.
											] ifFalse: [
												(char == $3) ifTrue: [
														"player 3"
														c := PlayerCell new.
														c color: Color green.
														game addPlayer: c WithId: 3.
													] ifFalse: [
														"otherwise: blank field"
														c := PacmanCell new.
													].
											].
									].
							].
					].
				
				"add newly created cell to matrix and add it to game"
				cells at: i at: j put: c.
				game cell: c At: j At: i.
				j := j+1.
			].
		i := i+1.
	].
	
	file close.
	
	^cells.! !

BorderedMorph subclass: #PacmanGame
	instanceVariableNames: 'cells players'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pacman'!

!PacmanGame class methodsFor: 'as yet unclassified' stamp: 'cl 6/19/2012 14:16'!
maxPlayers
	"returns number of maximum players"
	^3.! !

!PacmanGame methodsFor: 'logic' stamp: 'cl 6/19/2012 14:37'!
addPlayer: aPlayerCell WithId: anInteger
	"adds a player to this game, if not already exists
	returns trueif player was added, otherwise false"
	|retVal player|
	(anInteger > PacmanGame maxPlayers) ifTrue: [
			"Id is greater than max players"
			^false.
		].
	
	player := players at: 1 at: anInteger.
	player ifNotNil: [
			"player already in players, set new color"
			player color: aPlayerCell color.
			^false.
		].
	
	
	retVal := players columnCount < PacmanGame maxPlayers.
	retVal ifTrue: [
			players at: 1 at: anInteger put: aPlayerCell.
		]. 
	^retVal.! !

!PacmanGame methodsFor: 'eventhandler' stamp: 'cl 6/19/2012 13:47'!
btnStartPressed
	"eventhandler for button start click"
	|parser c|
	parser := WorldParser new addGame: self.
	self removeAllMorphs.
	cells := parser parseFileAt: 'game1.txt'.
	"get one cell"
	c := cells at: 1 at: 1.
	
	self height: (cells rowCount * c height) + (2 * self borderWidth).
	self width: (cells columnCount * c width) + (2 * self borderWidth).
	! !

!PacmanGame methodsFor: 'logic' stamp: 'cl 6/19/2012 10:59'!
cell: c At: i At: j
	"creates a new cell with given offset"
	|origin|
	origin := self innerBounds origin.
	self addMorph: c.
	c position: ((i-1) * c width) @ ((j-1) * c height) + origin.
	! !

!PacmanGame methodsFor: 'event handling' stamp: 'cl 6/19/2012 14:48'!
handlesKeyboard: anEvent
	^true.! !

!PacmanGame methodsFor: 'event handling' stamp: 'cl 6/19/2012 15:29'!
handlesMouseDown: anEvent
	^true.! !

!PacmanGame methodsFor: 'initialization' stamp: 'cl 6/19/2012 14:55'!
initialize
	|btnStart|
	super initialize.
	cells := nil.
	self bounds: (200@100 extent: (200@200)).
	self color: Color white.
	self borderColor: Color blue.
	
	players := Matrix rows: 1 columns: PacmanGame maxPlayers.
	
	btnStart := MyStartButton new game: self.
	self addMorph: btnStart.
	
	"give game keyboard focus"
	ActiveHand newKeyboardFocus: self.! !

!PacmanGame methodsFor: 'event handling' stamp: 'cl 6/19/2012 14:44'!
keyStroke: anEvent
	"key Stroke event"
	Transcript show: 'KEY STROKE'.! !

!PacmanGame methodsFor: 'event handling' stamp: 'cl 6/19/2012 15:35'!
mouseDown: anEvent
	"takes keyboardfocus if selected"
	ActiveHand newKeyboardFocus: self.
	super mouseDown: anEvent.! !

!PacmanGame methodsFor: 'logic' stamp: 'cl 6/19/2012 11:24'!
startGame
	"starts a new game"
	! !
